#!/usr/bin/perl
#
# This code is distributed under the terms of the GPL
#
# (c) 2006-2008 marco.s - http://update-accelerator.advproxy.net
#
# Portions (c) 2008 by dotzball - http://www.blockouttraffic.de
#
# $Id: updatexlrator.cgi,v 2.1.0 2008/07/16 00:00:00 marco.s Exp $
#
###############################################################################
#                                                                             #
# IPFire.org - A linux based firewall                                         #
# Copyright (C) 2005-2010  IPFire Team                                        #
#                                                                             #
# This program is free software: you can redistribute it and/or modify        #
# it under the terms of the GNU General Public License as published by        #
# the Free Software Foundation, either version 3 of the License, or           #
# (at your option) any later version.                                         #
#                                                                             #
# This program is distributed in the hope that it will be useful,             #
# but WITHOUT ANY WARRANTY; without even the implied warranty of              #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               #
# GNU General Public License for more details.                                #
#                                                                             #
# You should have received a copy of the GNU General Public License           #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.       #
#                                                                             #
###############################################################################
#
# Changelog:
# 2012-10-27: nightshift - Bugfix regarding showing wrong vendor icon while Download of new Updates
# 2012-10-27: nightshift - Optimizing logic of check for vendor icons
#

use strict;

# enable only the following on debugging purpose
use warnings; # no warnings 'once';# 'redefine', 'uninitialized';
use CGI::Carp 'fatalsToBrowser';

use IO::Socket;

require '/var/ipfire/general-functions.pl';
require "${General::swroot}/lang.pl";
require "${General::swroot}/header.pl";

my %color = ();
my %checked=();
my %selected=();
my %netsettings=();
my %mainsettings=();
my %proxysettings=();
my %xlratorsettings=();
my %dlinfo=();
my $id=0;
my @dfdata=();
my $dfstr='';
my $dudata='';
my $dustr='';
my @updatelist=();
my @sources=();
my $sourceurl='';
my $vendorid='';
my $uuid='';
my $status=0;
my $updatefile='';
my $cachefile='';
my $shortname='';
my $time='';
my $filesize=0;
my $filedate='';
my $lastaccess='';
my $lastcheck='';
my $cachedtraffic=0;
my @requests=();
my $data='';
my $counts=0;
my $numfiles=0;
my $cachehits=0;
my $efficiency='0.0';
my @vendors=();
my %vendorstats=();
my %vendimg = ();

my $repository = "/var/updatecache/";
my $webhome = "/srv/web/ipfire/html";
my $webimgdir = "/images/updbooster";

my $hintcolour = '#FFFFCC';
my $colourgray = '#808080';

my $sfUnknown='0';
my $sfOk='1';
my $sfOutdated='2';
my $sfNoSource='3';

my $not_accessed_last='';
my $errormessage='';

my @repositorylist=();
my @repositoryfiles=();
my @downloadlist=();
my @downloadfiles=();

my @metadata=();

&General::readhash("${General::swroot}/ethernet/settings", \%netsettings);
&General::readhash("${General::swroot}/main/settings", \%mainsettings);
&General::readhash("${General::swroot}/proxy/settings", \%proxysettings);
&General::readhash("/srv/web/ipfire/html/themes/".$mainsettings{'THEME'}."/include/colors.txt", \%color);

$xlratorsettings{'ACTION'} = '';
$xlratorsettings{'ENABLE_LOG'} = 'off';
$xlratorsettings{'PASSIVE_MODE'} = 'off';
$xlratorsettings{'MAX_DISK_USAGE'} = '75';
$xlratorsettings{'LOW_DOWNLOAD_PRIORITY'} = 'off';
$xlratorsettings{'MAX_DOWNLOAD_RATE'} = '';
$xlratorsettings{'ENABLE_AUTOCHECK'} = 'off';
$xlratorsettings{'FULL_AUTOSYNC'} = 'off';
$xlratorsettings{'NOT_ACCESSED_LAST'} = 'month1';
$xlratorsettings{'REMOVE_NOSOURCE'} = 'off';
$xlratorsettings{'REMOVE_OUTDATED'} = 'off';
$xlratorsettings{'REMOVE_OBSOLETE'} = 'off';
$xlratorsettings{'REMOVE_TODELETE'} = 'off';
$xlratorsettings{'TODELETE'} = 'off';

&Header::getcgihash(\%xlratorsettings);

###2012-11-25: nightshift
# Initialise vendor image hash

&initvendimg();

####

$xlratorsettings{'EXTENDED_GUI'} = '';

if ($xlratorsettings{'ACTION'} eq "$Lang::tr{'updxlrtr statistics'} >>")
	{ $xlratorsettings{'EXTENDED_GUI'} = 'statistics'; }

if ($xlratorsettings{'ACTION'} eq "$Lang::tr{'updxlrtr maintenance'} >>")
	{ $xlratorsettings{'EXTENDED_GUI'} = 'maintenance'; }

if ($xlratorsettings{'ACTION'} eq "$Lang::tr{'updxlrtr purge'}")
{
	$xlratorsettings{'EXTENDED_GUI'} = 'maintenance';

	if (  ($xlratorsettings{'REMOVE_OBSOLETE'} eq 'on')
	   || ($xlratorsettings{'REMOVE_NOSOURCE'} eq 'on')
	   || ($xlratorsettings{'REMOVE_OUTDATED'} eq 'on')
	   || ($xlratorsettings{'REMOVE_TODELETE'} eq 'on')
	   )
	{
		undef (@sources);
		undef @repositoryfiles;
		foreach (<$repository/*>)
		{
			if (-d $_)	{
					unless (/^$repository\/download$/) { push(@sources,$_); }
					}
		}

		foreach (@sources)
		{
			@updatelist=<$_/*>;
			$vendorid = substr($_,rindex($_,"/")+1);
			foreach(@updatelist) {
				$uuid = substr($_,rindex($_,"/")+1);
				if (-e "$_/source.url") {
					open (FILE,"$_/source.url");
					$sourceurl=<FILE>;
					close FILE;
					chomp($sourceurl);
					$updatefile = substr($sourceurl,rindex($sourceurl,'/')+1,length($sourceurl));
					$updatefile = "$vendorid/$uuid/$updatefile";
					push(@repositoryfiles,$updatefile);
				}
			}
		}

		foreach (@repositoryfiles)
		{
			($vendorid,$uuid,$updatefile) = split('/');

			if (-e "$repository/$vendorid/$uuid/status") {
				open (FILE,"$repository/$vendorid/$uuid/status");
				@metadata = <FILE>;
				close FILE;
				chomp(@metadata);
				$status = $metadata[-1];
			}

			if (-e "$repository/$vendorid/$uuid/access.log") {
				open (FILE,"$repository/$vendorid/$uuid/access.log");
				@metadata = <FILE>;
				close FILE;
				chomp(@metadata);
				$lastaccess = $metadata[-1];
			}

			if (($xlratorsettings{'REMOVE_NOSOURCE'} eq 'on') && ($status == $sfNoSource))
				{ if (-e "$repository/$vendorid/$uuid/$updatefile") { system("rm -r $repository/$vendorid/$uuid"); } }
			if (($xlratorsettings{'REMOVE_OUTDATED'} eq 'on') && ($status == $sfOutdated))
				{ if (-e "$repository/$vendorid/$uuid/$updatefile") { system("rm -r $repository/$vendorid/$uuid"); } }
			if ($xlratorsettings{'REMOVE_OBSOLETE'} eq 'on')
			{
				if (($xlratorsettings{'NOT_ACCESSED_LAST'} eq 'week') && ($lastaccess < (time - 604800)))
					{ if (-e "$repository/$vendorid/$uuid/$updatefile") { system("rm -r $repository/$vendorid/$uuid"); } }
				elsif (($xlratorsettings{'NOT_ACCESSED_LAST'} eq 'month1') && ($lastaccess < (time - 2505600)))
					{ if (-e "$repository/$vendorid/$uuid/$updatefile") { system("rm -r $repository/$vendorid/$uuid"); } }
				elsif (($xlratorsettings{'NOT_ACCESSED_LAST'} eq 'month3') && ($lastaccess < (time - 7516800)))
					{ if (-e "$repository/$vendorid/$uuid/$updatefile") { system("rm -r $repository/$vendorid/$uuid"); } }
				elsif (($xlratorsettings{'NOT_ACCESSED_LAST'} eq 'month6') && ($lastaccess < (time - 15033600)))
					{ if (-e "$repository/$vendorid/$uuid/$updatefile") { system("rm -r $repository/$vendorid/$uuid"); } }
				elsif (($xlratorsettings{'NOT_ACCESSED_LAST'} eq 'year') && ($lastaccess < (time - 31536000)))
					{ if (-e "$repository/$vendorid/$uuid/$updatefile") { system("rm -r $repository/$vendorid/$uuid"); } }
			}

			if (($xlratorsettings{'REMOVE_TODELETE'} eq 'on') && ($xlratorsettings{'TODELETE'} ne 'off'))
			{
				my @todelete = split(/\|/, $xlratorsettings{'TODELETE'});
				foreach (@todelete) {
					unless ($_ =~ /^download\//) {
						($vendorid,$uuid,$cachefile) = split(/\//, $_);
						if (-e "$repository/$vendorid/$uuid/$cachefile") { system("rm -r $repository/$vendorid/$uuid"); }
					}
				}
			}
		}
	}
}

if ($xlratorsettings{'ACTION'} eq $Lang::tr{'save'}) {
	if (!($xlratorsettings{'MAX_DISK_USAGE'} =~ /^\d+$/)
		|| ($xlratorsettings{'MAX_DISK_USAGE'} < 1)
		|| ($xlratorsettings{'MAX_DISK_USAGE'} > 100)) {
		$errormessage = $Lang::tr{'updxlrtr invalid disk usage'};
		goto ERROR;
	}
	if (($xlratorsettings{'MAX_DOWNLOAD_RATE'} ne '') && ((!($xlratorsettings{'MAX_DOWNLOAD_RATE'} =~ /^\d+$/))
		|| ($xlratorsettings{'MAX_DOWNLOAD_RATE'} < 1))) {
		$errormessage = $Lang::tr{'updxlrtr invalid download rate'};
		goto ERROR;
	}

	&savesettings;
}

if ($xlratorsettings{'ACTION'} eq $Lang::tr{'updxlrtr save and restart'}) {
	if (!($xlratorsettings{'MAX_DISK_USAGE'} =~ /^\d+$/)
		|| ($xlratorsettings{'MAX_DISK_USAGE'} < 1)
		|| ($xlratorsettings{'MAX_DISK_USAGE'} > 100)) {
		$errormessage = $Lang::tr{'updxlrtr invalid disk usage'};
		goto ERROR;
	}
	if (($xlratorsettings{'MAX_DOWNLOAD_RATE'} ne '') && ((!($xlratorsettings{'MAX_DOWNLOAD_RATE'} =~ /^\d+$/))
		|| ($xlratorsettings{'MAX_DOWNLOAD_RATE'} < 1))) {
		$errormessage = $Lang::tr{'updxlrtr invalid download rate'};
		goto ERROR;
	}
	if ((!(-e "${General::swroot}/proxy/enable")) && (!(-e "${General::swroot}/proxy/enable_blue"))) {
		$errormessage = $Lang::tr{'updxlrtr web proxy service required'};
		goto ERROR;
	}
	if (!($proxysettings{'ENABLE_UPDXLRATOR'} eq 'on')) {
		$errormessage = $Lang::tr{'updxlrtr not enabled'};
		goto ERROR;
	}

	&savesettings;
	system('/usr/local/bin/squidctrl restart > /dev/null 2>&1');
}


if ($xlratorsettings{'ACTION'} eq $Lang::tr{'updxlrtr cancel download'}) {
	$updatefile = $xlratorsettings{'ID'};
	if ($updatefile =~ /^download\//) 	{
		($uuid,$vendorid,$updatefile) = split('/',$updatefile);
		if (-e "$repository/download/$vendorid/$updatefile.info") {
			&General::readhash("$repository/download/$vendorid/$updatefile.info", \%dlinfo);
			$id = &getPID("\\s${General::swroot}/updatexlrator/bin/download\\s.*\\s".quotemeta($dlinfo{'SRCURL'})."\\s\\d\\s\\d\$");
			if ($id) { system("/bin/kill -9 $id"); }
			$id = &getPID("\\s/usr/bin/wget\\s.*\\s".quotemeta($dlinfo{'SRCURL'})."\$");
			if ($id) { system("/bin/kill -9 $id"); }
			system("rm $repository/download/$vendorid/$updatefile.info");
		}
		if (-e "$repository/download/$vendorid/$updatefile") {
			system("rm $repository/download/$vendorid/$updatefile");
		}
	}
}

$not_accessed_last = $xlratorsettings{'NOT_ACCESSED_LAST'};
undef($xlratorsettings{'NOT_ACCESSED_LAST'});

if (-e "${General::swroot}/updatexlrator/settings") {
	&General::readhash("${General::swroot}/updatexlrator/settings", \%xlratorsettings);
}

if ($xlratorsettings{'NOT_ACCESSED_LAST'} eq '') {
	$xlratorsettings{'NOT_ACCESSED_LAST'} = $not_accessed_last;
}

ERROR:

$checked{'ENABLE_LOG'}{'off'} = '';
$checked{'ENABLE_LOG'}{'on'} = '';
$checked{'ENABLE_LOG'}{$xlratorsettings{'ENABLE_LOG'}} = "checked='checked'";
$checked{'PASSIVE_MODE'}{'off'} = '';
$checked{'PASSIVE_MODE'}{'on'} = '';
$checked{'PASSIVE_MODE'}{$xlratorsettings{'PASSIVE_MODE'}} = "checked='checked'";
$checked{'LOW_DOWNLOAD_PRIORITY'}{'off'} = '';
$checked{'LOW_DOWNLOAD_PRIORITY'}{'on'} = '';
$checked{'LOW_DOWNLOAD_PRIORITY'}{$xlratorsettings{'LOW_DOWNLOAD_PRIORITY'}} = "checked='checked'";
$checked{'ENABLE_AUTOCHECK'}{'off'} = '';
$checked{'ENABLE_AUTOCHECK'}{'on'} = '';
$checked{'ENABLE_AUTOCHECK'}{$xlratorsettings{'ENABLE_AUTOCHECK'}} = "checked='checked'";
$checked{'FULL_AUTOSYNC'}{'off'} = '';
$checked{'FULL_AUTOSYNC'}{'on'} = '';
$checked{'FULL_AUTOSYNC'}{$xlratorsettings{'FULL_AUTOSYNC'}} = "checked='checked'";
$checked{'REMOVE_NOSOURCE'}{'off'} = '';
$checked{'REMOVE_NOSOURCE'}{'on'} = '';
$checked{'REMOVE_NOSOURCE'}{$xlratorsettings{'REMOVE_NOSOURCE'}} = "checked='checked'";
$checked{'REMOVE_OUTDATED'}{'off'} = '';
$checked{'REMOVE_OUTDATED'}{'on'} = '';
$checked{'REMOVE_OUTDATED'}{$xlratorsettings{'REMOVE_OUTDATED'}} = "checked='checked'";
$checked{'REMOVE_OBSOLETE'}{'off'} = '';
$checked{'REMOVE_OBSOLETE'}{'on'} = '';
$checked{'REMOVE_OBSOLETE'}{$xlratorsettings{'REMOVE_OBSOLETE'}} = "checked='checked'";
$checked{'REMOVE_TODELETE'}{'off'} = '';
$checked{'REMOVE_TODELETE'}{'on'} = '';
$checked{'REMOVE_TODELETE'}{$xlratorsettings{'REMOVE_TODELETE'}} = "checked='checked'";
$checked{'TODELETE'}{'off'} = '';
$checked{'TODELETE'}{'on'} = '';
$checked{'TODELETE'}{$xlratorsettings{'TODELETE'}} = "checked='checked'";
$selected{'AUTOCHECK_SCHEDULE'}{'daily'} = '';
$selected{'AUTOCHECK_SCHEDULE'}{'weekly'} = '';
$selected{'AUTOCHECK_SCHEDULE'}{'monthly'} = '';
$selected{'AUTOCHECK_SCHEDULE'}{$xlratorsettings{'AUTOCHECK_SCHEDULE'}} = "selected='selected'";
$selected{'NOT_ACCESSED_LAST'}{'week'} = '';
$selected{'NOT_ACCESSED_LAST'}{'month1'} = '';
$selected{'NOT_ACCESSED_LAST'}{'month3'} = '';
$selected{'NOT_ACCESSED_LAST'}{'month6'} = '';
$selected{'NOT_ACCESSED_LAST'}{'year'} = '';
$selected{'NOT_ACCESSED_LAST'}{$xlratorsettings{'NOT_ACCESSED_LAST'}} = "selected='selected'";

# ----------------------------------------------------
#    Settings dialog
# ----------------------------------------------------

&Header::showhttpheaders();
&Header::openpage($Lang::tr{'updxlrtr configuration'}, 1, '<link rel=\'stylesheet\' type=\'text/css\' href=\'/themes/'.$mainsettings{'THEME'}.'/include/upxlr.css\' />' );
&Header::openbigbox('100%', 'left', '', $errormessage);

if ($errormessage) {
	&Header::openbox('100%', 'left', $Lang::tr{'error messages'});
	print "$errormessage&nbsp;\n";
	&Header::closebox();
}

&Header::openbox('100%', 'left', $Lang::tr{'updxlrtr update accelerator'});

print <<END
END
;

print <<END
<form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>
<fieldset>
<legend>$Lang::tr{'updxlrtr common settings'}</legend>
<input id='check01' type='checkbox' name='ENABLE_LOG' $checked{'ENABLE_LOG'}{'on'} />
<label for='check01'>$Lang::tr{'updxlrtr enable log'}</label>
<input id='check02' type='checkbox' name='PASSIVE_MODE' $checked{'PASSIVE_MODE'}{'on'} />
<label for='check02'>$Lang::tr{'updxlrtr passive mode'}</label>
<label for='text01'>$Lang::tr{'updxlrtr max disk usage'} (%):</label>
<input id='text01' type='text' name='MAX_DISK_USAGE' value='$xlratorsettings{'MAX_DISK_USAGE'}' size='1' />
</fieldset>
<fieldset>
<legend>$Lang::tr{'updxlrtr performance options'}</legend>
<input id='check04' type='checkbox' name='LOW_DOWNLOAD_PRIORITY' $checked{'LOW_DOWNLOAD_PRIORITY'}{'on'} />
<label for='check04'>$Lang::tr{'updxlrtr low download priority'}</label>
<label for='text02' class='icoblob'>$Lang::tr{'updxlrtr max download rate'}:</label>
<input id='text02' type='text' name='MAX_DOWNLOAD_RATE' title='*' value='$xlratorsettings{'MAX_DOWNLOAD_RATE'}' size='5' />
</fieldset>
<fieldset>
<legend>$Lang::tr{'updxlrtr source checkup'}</legend>
<input id='check06' type='checkbox' name='ENABLE_AUTOCHECK' $checked{'ENABLE_AUTOCHECK'}{'on'} />
<label for='check06'>$Lang::tr{'updxlrtr enable autocheck'}</label>
<label for='select01'>$Lang::tr{'updxlrtr source checkup schedule'}:</label>
<select id='select01' name='AUTOCHECK_SCHEDULE'>
<option value='daily' $selected{'AUTOCHECK_SCHEDULE'}{'daily'}>$Lang::tr{'updxlrtr daily'}</option>
<option value='weekly' $selected{'AUTOCHECK_SCHEDULE'}{'weekly'}>$Lang::tr{'updxlrtr weekly'}</option>
<option value='monthly' $selected{'AUTOCHECK_SCHEDULE'}{'monthly'}>$Lang::tr{'updxlrtr monthly'}</option>
</select>
<br />
<input id='check07' type='checkbox' name='FULL_AUTOSYNC' $checked{'FULL_AUTOSYNC'}{'on'} />
<label for='check07'>$Lang::tr{'updxlrtr full autosync'}</label>
</fieldset>
<fieldset style='border:0'>
<input type='submit' name='ACTION' value='$Lang::tr{'save'}' />
<input type='submit' name='ACTION' value='$Lang::tr{'updxlrtr save and restart'}' />
END
;
print"<input type='submit' name='ACTION' value='$Lang::tr{'updxlrtr statistics'}";
if ($xlratorsettings{'EXTENDED_GUI'} eq 'statistics') { print " &lt;&lt;' />"; } else { print " &gt;&gt;' />\n"; }
print"<input type='submit' name='ACTION' value='$Lang::tr{'updxlrtr maintenance'}";
if ($xlratorsettings{'EXTENDED_GUI'} eq 'maintenance') { print " &lt;&lt;' />"; } else { print " &gt;&gt;' />\n"; }
print <<END
</fieldset>
</form>
END
;
&Header::closebox();

# ----------------------------------------------------
#   List pending downloads - if any
# ----------------------------------------------------

if (($xlratorsettings{'EXTENDED_GUI'} ne 'statistics') && ($xlratorsettings{'EXTENDED_GUI'} ne 'maintenance'))
{
	@downloadlist = <$repository/download/*>;

	undef(@downloadfiles);
	foreach (@downloadlist)
	{
		if (-d)
		{
			my @filelist = <$_/*>;
			$vendorid = substr($_,rindex($_,"/")+1);
			foreach(@filelist) {
				next if(/\.info$/);
				$updatefile = substr($_,rindex($_,"/")+1);
				$updatefile .= ":download/$vendorid/$updatefile";
				$updatefile = " ".$updatefile;
				push(@downloadfiles, $updatefile);
			}
		}
	}

	if (@downloadfiles)
	{
		&Header::openbox('100%', 'left', "$Lang::tr{'updxlrtr pending downloads'}");
		print <<END
<table width='100%' id='listuploads'>
<caption>$Lang::tr{'updxlrtr current downloads'}</caption>
<thead>
<tr>
	<th>$Lang::tr{'status'}</th>
	<th>$Lang::tr{'updxlrtr source'}</th>
	<th>$Lang::tr{'updxlrtr filename'}</th>
	<th>$Lang::tr{'updxlrtr filesize'}</th>
	<th>$Lang::tr{'date'}</th>
	<th>$Lang::tr{'updxlrtr progress'}</th>
	<th>&nbsp;</th>
</tr>
</thead>
<tfoot>
<tr>
	<td colspan='7' id='legenddownload'>
END
;
	&printlegenddownload();
print <<END
	</td>
</tr>
</tfoot>
<tbody>
END
;
	$id = 0;
	foreach $updatefile (@downloadfiles)
	{
		$updatefile =~ s/.*://;
		my $size_updatefile = 0;
		my $mtime = 0;
		if(-e "$repository/$updatefile") {
			$size_updatefile = (-s "$repository/$updatefile");
			$mtime = &getmtime("$repository/$updatefile");
		}
		if (-e "$repository/$updatefile.info") {
			&General::readhash("$repository/$updatefile.info", \%dlinfo);
		}
		else { undef(%dlinfo); }
		print "<tr>\n";
		$filesize = $size_updatefile;
		1 while $filesize =~ s/^(-?\d+)(\d{3})/$1.$2/;
		my ($SECdt,$MINdt,$HOURdt,$DAYdt,$MONTHdt,$YEARdt) = localtime($mtime);
		$DAYdt   = sprintf ("%.02d",$DAYdt);
		$MONTHdt = sprintf ("%.02d",$MONTHdt+1);
		$YEARdt  = sprintf ("%.04d",$YEARdt+1900);
		$filedate = $DAYdt.".".$MONTHdt.".".$YEARdt;
		($uuid,$vendorid,$shortname) = split('/',$updatefile);
		if (&getPID("\\s/usr/bin/wget\\s.*\\s".quotemeta($dlinfo{'SRCURL'})."\$"))
			 { print "\t<td class='ledblue' title='$Lang::tr{'updxlrtr condition download'}'></td>\n"; }
		else { print "\t<td class='ledgrey' title='$Lang::tr{'updxlrtr condition suspended'}'></td>\n"; }
		if ($vendimg{$vendorid})
			 { print "\t<td class='left vendimg' style='background: url(".$vendimg{$vendorid}.") 4px no-repeat;' title='".ucfirst $vendorid."'>".ucfirst $dlinfo{'VENDORID'}."</td>\n"; }
		else { print "\t<td img class='left vendimg' style='background: url(".$vendimg{'unknown'}.") 4px no-repeat;' title='".ucfirst $vendorid."'>".ucfirst $dlinfo{'VENDORID'}."</td>\n"; }
		$shortname = substr($updatefile,rindex($updatefile,"/")+1);
		$shortname =~ s/(.*)_[\da-f]*(\.(exe|cab|psf)$)/$1_*$2/i;
		$filesize = $dlinfo{'REMOTESIZE'};
		1 while $filesize =~ s/^(-?\d+)(\d{3})/$1.$2/;
		$dlinfo{'VENDORID'} = ucfirst $vendorid;
		print <<END
	<td title='cache:/$updatefile'>$shortname</td>
	<td>$filesize</td>
	<td>$filedate</td>
	<td>
END
;
		my $percent = '0%';
		if ($dlinfo{'REMOTESIZE'} && $size_updatefile)
			{ $percent = int(100 / ($dlinfo{'REMOTESIZE'} / $size_updatefile)).'%'; }
		print &percentbar($percent)."</td>";
print <<END
	<td align='center'><form method='post' id='frma$id' action='$ENV{'SCRIPT_NAME'}'>
		<button type='submit' class='btndelete' name='$Lang::tr{'updxlrtr cancel download'}'  title='$Lang::tr{'updxlrtr cancel download'}' />
		<input type='hidden' name='ID' value='$updatefile' />
		<input type='hidden' name='ACTION' value='$Lang::tr{'updxlrtr cancel download'}' />
	</form></td>
</tr>
END
;
	$id += 1;
	}
	print <<END
	</tbody>
	</table>
END
;
	&Header::closebox();
	}
}
# =====================================================================================
#  CACHE STATISTICS
# =====================================================================================

if ($xlratorsettings{'EXTENDED_GUI'} eq 'statistics') {

# ----------------------------------------------------
#    Get statistics
# ----------------------------------------------------

	@sources=();
	foreach (<$repository/*>)
	{
		if (-d $_) {
			unless ((/^$repository\/download$/) || (/^$repository\/lost\+found$/)) { push(@sources,$_); }
		}
	}

	@vendors=();
	foreach (@sources)
	{
		$vendorid=substr($_,rindex($_,'/')+1,length($_));
		push(@vendors,$vendorid);
		$vendorstats{$vendorid."_filesize"} = 0;
		$vendorstats{$vendorid."_requests"} = 0;
		$vendorstats{$vendorid."_files"} = 0;
		$vendorstats{$vendorid."_cachehits"} = 0;
		$vendorstats{$vendorid."_0"} = 0;
		$vendorstats{$vendorid."_1"} = 0;
		$vendorstats{$vendorid."_2"} = 0;
		$vendorstats{$vendorid."_3"} = 0;
		@updatelist=<$_/*>;
		foreach $data (@updatelist) {
			if (-e "$data/source.url") {
				open (FILE,"$data/source.url");
				$sourceurl=<FILE>;
				close FILE;
				chomp($sourceurl);
				$updatefile = substr($sourceurl,rindex($sourceurl,'/')+1,length($sourceurl));
				my $size_updatefile = 0;
				if(-e "$data/$updatefile") {
					$size_updatefile = (-s "$data/$updatefile");
				}
				else {
					# DEBUG
					#die "file not found: $data/$updatefile\n";
				}
				#
				# Total file size
				#
				$filesize += $size_updatefile;
				#
				# File size for this source
				#
				$vendorstats{$vendorid."_filesize"} += $size_updatefile;
				#
				# Number of requests from cache for this source
				#
				open (FILE,"$data/access.log");
				@requests=<FILE>;
				close FILE;
				chomp(@requests);
				$counts = @requests;
				$counts--;
				$vendorstats{$vendorid."_requests"} += $counts;
				$cachehits += $counts;
				#
				# Total number of files in cache
				#
				$numfiles++;
				#
				# Number of files for this source
				#
				$vendorstats{$vendorid."_files"}++;
				#
				# Count cache status occurences
				#
				open (FILE,"$data/status");
				$_=<FILE>;
				close FILE;
				chomp;
				$vendorstats{$vendorid."_".$_}++;
				#
				# Calculate cached traffic for this source
				#
				$vendorstats{$vendorid."_cachehits"} += $counts * $size_updatefile;
				#
				# Calculate total cached traffic
				#
				$cachedtraffic += $counts * $size_updatefile;
			}
		}
	}

	if ($numfiles) { $efficiency = sprintf("%.1f", $cachehits / $numfiles); }

	1 while $filesize =~ s/^(-?\d+)(\d{3})/$1.$2/;
	1 while $cachedtraffic =~ s/^(-?\d+)(\d{3})/$1.$2/;

# ----------------------------------------------------
#    Show statistics
# ----------------------------------------------------

	&Header::openbox('100%', 'left', "$Lang::tr{'updxlrtr cache statistics'}");
	unless ($numfiles) { print "$Lang::tr{'updxlrtr empty repository'}\n<hr />\n"; }
	&printdiskusage($Lang::tr{'updxlrtr disk usage'},'statdiskusage');

	if ($numfiles) {
		print <<END
<hr />
<br />
<table width='100%' id='summary'>
<caption>$Lang::tr{'updxlrtr summary'}</caption>
<tbody>
<tr>
	<td>$Lang::tr{'updxlrtr total files'}:</td>
	<td><span style='color:$colourgray'>$numfiles</span></td>
	<td>$Lang::tr{'updxlrtr total cache size'}:</td>
	<td align='right'><span style='color:$colourgray'>$filesize</span></td>
	<td></td>
</tr>
<tr>
	<td>$Lang::tr{'updxlrtr efficiency index'}:</td>
	<td><span style='color:$colourgray'>$efficiency</span></td>
	<td>$Lang::tr{'updxlrtr total data from cache'}:</td>
	<td align='right'><span style='color:$colourgray'>$cachedtraffic</span></td>
	<td></td>
</tr>
</tbody>
</table>
<hr />
<table width='100%' id='liststatbysrc'>
<caption>$Lang::tr{'updxlrtr statistics by source'}</caption>
<colgroup>
	<col span='2' style='width: 20px'></col>
	<col></col>
	<col></col>
	<col span='4' style='width: 40px'></col>
</colgroup>

<thead>
<tr>
	<th>$Lang::tr{'updxlrtr source'}</th>
	<th>$Lang::tr{'updxlrtr files'}</th>
	<th>$Lang::tr{'updxlrtr cache size'}</th>
	<th>$Lang::tr{'updxlrtr data from cache'}</th>
	<th class='ledgreen' title='$Lang::tr{'updxlrtr condition ok'}'></th>
	<th class='ledyellow' title='$Lang::tr{'updxlrtr condition nosource'}'></th>
	<th class='ledred' title='$Lang::tr{'updxlrtr condition outdated'}'></th>
	<th class='ledgrey' title='$Lang::tr{'updxlrtr condition unknown'}'></th>
</tr>
</thead>
<tfoot>
<tr>
	<td colspan='8' id='legendstatsstatus'>
END
;
	&printlegendstatus();
	print <<END
</td>
</tr>
</tfoot>
<tbody>
END
;

	foreach (@vendors) {
		$vendorid = $_;
		unless ($vendorstats{$vendorid . "_files"}) { next; }
		print "<tr>\n";
		if ($vendimg{$vendorid}) {
			print "\t<td class='left vendimg' style='background: url(".$vendimg{$vendorid}.") 4px no-repeat;' title='".ucfirst $vendorid."'>". ucfirst $vendorid ."</td>\n";
		}
		else { print "\t<td class='left vendimg' style='background: url(".$vendimg{'unknown'}.") 4px no-repeat;' title='".ucfirst $vendorid."'>". ucfirst $vendorid ."</td>\n"; }
		print "\t<td>";
		printf "%5d", $vendorstats{$vendorid."_files"};
		print "</td>\n";
		unless ($vendorstats{$vendorid."_filesize"}) { $vendorstats{$vendorid."_filesize"} = '0'; }
		1 while $vendorstats{$vendorid."_filesize"} =~ s/^(-?\d+)(\d{3})/$1.$2/;
		print "<td>";
		printf "%15s", $vendorstats{$vendorid."_filesize"};
		print "</td>\n";
		unless ($vendorstats{$vendorid."_cachehits"}) { $vendorstats{$vendorid."_cachehits"} = '0'; }
		1 while $vendorstats{$vendorid."_cachehits"} =~ s/^(-?\d+)(\d{3})/$1.$2/;
		print "\t<td>";
		printf "%15s", $vendorstats{$vendorid."_cachehits"};
		print "</td>\n";
		print "\t<td>";
		printf "%5d", $vendorstats{$vendorid."_1"};
		print "</td>\n";
		print "\t<td>";
		printf "%5d", $vendorstats{$vendorid."_3"};
		print "</td>\n";
		print "\t<td>";
		printf "%5d", $vendorstats{$vendorid."_2"};
		print "</td>\n";
		print "\t<td>";
		printf "%5d", $vendorstats{$vendorid."_0"};
		print "</td>\n";
		print "</tr>\n";
	}
	print "</tbody>\n";
	print "</table>\n";
}
&Header::closebox();

} # End if statistics

# =====================================================================================
#  CACHE MAINTENANCE
# =====================================================================================

if ($xlratorsettings{'EXTENDED_GUI'} eq 'maintenance') {


# ----------------------------------------------------
#    File list dialog
# ----------------------------------------------------

	&Header::openbox('100%', 'left', "$Lang::tr{'updxlrtr cache maintenance'}");
	@sources = <$repository/download/*>;
	undef @repositoryfiles;
	foreach (@sources) {
		if (-d) {
			@updatelist = <$_/*>;
			$vendorid = substr($_,rindex($_,"/")+1);
			foreach(@updatelist) {
				next if(/\.info$/);
				$updatefile = substr($_,rindex($_,"/")+1);
				$updatefile .= ":download/$vendorid/$updatefile";
				$updatefile = " ".$updatefile;
				push(@repositoryfiles,$updatefile);
			}
		}
	}

	undef (@sources);
	foreach (<$repository/*>) {
		if (-d $_) {
			unless (/^$repository\/download$/) { push(@sources,$_); }
		}
	}

	foreach (@sources) {
		@updatelist=<$_/*>;
		$vendorid = substr($_,rindex($_,"/")+1);
		foreach(@updatelist) {
			$uuid = substr($_,rindex($_,"/")+1);
			if (-e "$_/source.url") {
				open (FILE,"$_/source.url");
				$sourceurl=<FILE>;
				close FILE;
				chomp($sourceurl);
				$updatefile = substr($sourceurl,rindex($sourceurl,'/')+1,length($sourceurl));
				$_ = $updatefile; tr/[A-Z]/[a-z]/;
				$updatefile = "$_:$vendorid/$uuid/$updatefile";
				push(@repositoryfiles,$updatefile);
			}
		}
	}

	@repositoryfiles = sort(@repositoryfiles);
	unless (@repositoryfiles) { print "$Lang::tr{'updxlrtr empty repository'}\n<hr />\n"; }
	&printdiskusage ($Lang::tr{'updxlrtr disk usage'}, 'maintenancediskusage');
	if (@repositoryfiles) {
		print <<END
	<hr />
	<div class='post'>
	<form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>
	<fieldset>
	<legend>$Lang::tr{'updxlrtr all files'}</legend>
	<input id='check08' type='checkbox' name='REMOVE_OBSOLETE' $checked{'REMOVE_OBSOLETE'}{'on'} />
	<label for='check08'>$Lang::tr{'updxlrtr not accessed'}:</label>
	<select id='select02' name='NOT_ACCESSED_LAST'>
		<option value='week'   $selected{'NOT_ACCESSED_LAST'}{'week'}>$Lang::tr{'updxlrtr week'}</option>
		<option value='month1' $selected{'NOT_ACCESSED_LAST'}{'month1'}>$Lang::tr{'updxlrtr month'}</option>
		<option value='month3' $selected{'NOT_ACCESSED_LAST'}{'month3'}>$Lang::tr{'updxlrtr 3 months'}</option>
		<option value='month6' $selected{'NOT_ACCESSED_LAST'}{'month6'}>$Lang::tr{'updxlrtr 6 months'}</option>
		<option value='year'   $selected{'NOT_ACCESSED_LAST'}{'year'}>$Lang::tr{'updxlrtr year'}</option>
	</select>
	</fieldset>
	<fieldset>
	<legend>$Lang::tr{'updxlrtr marked as'} ...</legend>
	<input id='check09' type='checkbox' name='REMOVE_NOSOURCE' $checked{'REMOVE_NOSOURCE'}{'on'} />
	<label for='check09' class='ledyellow'>$Lang::tr{'updxlrtr condition nosource'}</label>
	<input id='check10' type='checkbox' name='REMOVE_OUTDATED' $checked{'REMOVE_OUTDATED'}{'on'} />
	<label for='check10' class='ledred'>$Lang::tr{'updxlrtr condition outdated'}</label>
	<input id='check11' type='checkbox' name='REMOVE_TODELETE' />
	<label for='check11' class='icodelete'>$Lang::tr{'updxlrtr remove file'}</label>
	</fieldset>
	<fieldset style='border:0'>
	<input type='submit' name='ACTION' value='$Lang::tr{'updxlrtr purge'}' />
	</fieldset>
	<hr />
END
;
		&printcurrentfiles($Lang::tr{'updxlrtr current files'}, @repositoryfiles);
	}
	print "</form>\n</div>\n";
	&Header::closebox();
}

# =====================================================================================

&Header::closebigbox();
&Header::closepage();

# -------------------------------------------------------------------

sub printcurrentfiles
{
	my $title = shift;
	my @files = @_;

	print <<END
<table id='listcurrfiles'>
<caption>$title</caption>
<colgroup>
	<col style='width: 20px'></col>
	<col style='width: 20px'></col>
	<col style='width: 0*'></col>
	<col span='4' style='width: 5%'></col>
	<col style='width: 20px'></col>
</colgroup>
<thead>
<tr>
	<th class='left'>$Lang::tr{'status'}</th>
	<th >$Lang::tr{'updxlrtr source'}</th>
	<th>$Lang::tr{'updxlrtr filename'}</th>
	<th>$Lang::tr{'updxlrtr filesize'}</th>
	<th>$Lang::tr{'date'}</th>
	<th class='icoreload' title='$Lang::tr{'updxlrtr last access'}'></th>
	<th class='icoglobe' title='$Lang::tr{'updxlrtr last checkup'}'></th>
	<th class='icodelete' title='$Lang::tr{'updxlrtr remove file'}'></th>
</tr>
</thead>
<tfoot>
<tr>
	<td colspan='9' id='legendicons'>
END
;
&printlegendicons();
print <<END
</td>
</tr>
<tr>
	<td colspan='9' id='legendstatus'>
END
;
&printlegendstatus();
print <<END
</td>
</tr>
<tr>
	<td colspan='9' id='legendsource'>
END
;
&printlegendsource();
print <<END
</td>
</tr>
</tfoot>
<tbody>
END
;
	$id = 0;
	foreach $updatefile (@files)
	{
		$updatefile =~ s/.*://;
		my $size_updatefile = 0;
		my $mtime = 0;
		if(-e "$repository/$updatefile") {
			$size_updatefile = (-s "$repository/$updatefile");
			$mtime = &getmtime("$repository/$updatefile");
		}
		print "<tr>\n";
		$filesize = $size_updatefile;
		1 while $filesize =~ s/^(-?\d+)(\d{3})/$1.$2/;
		my ($SECdt,$MINdt,$HOURdt,$DAYdt,$MONTHdt,$YEARdt) = localtime($mtime);
		$DAYdt   = sprintf ("%.02d",$DAYdt);
		$MONTHdt = sprintf ("%.02d",$MONTHdt+1);
		$YEARdt  = sprintf ("%.04d",$YEARdt+1900);
		$filedate = $DAYdt.".".$MONTHdt.".".$YEARdt;
		$lastaccess = "n/a";
		$lastcheck  = "n/a";
		$status = $sfUnknown;
		unless ($updatefile =~ /^download\//) {
			($vendorid,$uuid,$shortname) = split('/',$updatefile);
			if (-e "$repository/$vendorid/$uuid/access.log") {
				open (FILE,"$repository/$vendorid/$uuid/access.log");
				@metadata = <FILE>;
				close(FILE);
				chomp @metadata;
				($SECdt,$MINdt,$HOURdt,$DAYdt,$MONTHdt,$YEARdt) = localtime($metadata[-1]);
				$DAYdt   = sprintf ("%.02d",$DAYdt);
				$MONTHdt = sprintf ("%.02d",$MONTHdt+1);
				$YEARdt  = sprintf ("%.04d",$YEARdt+1900);
				if (($metadata[-1] =~ /^\d+/) && ($metadata[-1] >= 1))
					{ $lastaccess = $DAYdt.".".$MONTHdt.".".$YEARdt; }
			}
			if (-e "$repository/$vendorid/$uuid/checkup.log") {
				open (FILE,"$repository/$vendorid/$uuid/checkup.log");
				@metadata = <FILE>;
				close(FILE);
				chomp @metadata;
				($SECdt,$MINdt,$HOURdt,$DAYdt,$MONTHdt,$YEARdt) = localtime($metadata[-1]);
				$DAYdt = sprintf ("%.02d",$DAYdt);
				$MONTHdt = sprintf ("%.02d",$MONTHdt+1);
				$YEARdt  = sprintf ("%.04d",$YEARdt+1900);
				if (($metadata[-1] =~ /^\d+/) && ($metadata[-1] >= 1))
					{ $lastcheck = $DAYdt.".".$MONTHdt.".".$YEARdt; }
			}
			if (-e "$repository/$vendorid/$uuid/status") {
				open (FILE,"$repository/$vendorid/$uuid/status");
				@metadata = <FILE>;
				close(FILE);
				chomp @metadata;
				$status = $metadata[-1];
			}
		}
		else {
			($uuid,$vendorid,$shortname) = split('/',$updatefile);
			$status = $sfOutdated;
		}

		if ($status == $sfUnknown)
			{ print "\t\t<td><div class='ledgrey' title='$Lang::tr{'updxlrtr condition unknown'}'></div></td>\n"; }
		elsif ($status == $sfOk)
			{ print "\t\t<td><div class='ledgreen' title='$Lang::tr{'updxlrtr condition ok'}'></div></td>\n"; }
		elsif ($status == $sfNoSource)
			{ print "\t\t<td><div class='ledyellow' title='$Lang::tr{'updxlrtr condition nosource'}'></div></td>\n"; }
		elsif (($status == $sfOutdated) && (!($updatefile =~ /^download\//i)))
			{ print "\t\t<td><div class='ledred' title='$Lang::tr{'updxlrtr condition outdated'}'></div></td>\n"; }
		elsif (($status == $sfOutdated) && ($updatefile =~ /^download\//i))
			{ print "\t\t<td><div class='ledblue' title='$Lang::tr{'updxlrtr condition download'}'></div></td>\n"; }

#### With Vendorname behind image, nevertheless there is still the title-tag ...
#		if ($vendimg{$vendorid}) { 
#			print "\t\t<td class='left vendimg' style='background: url(".$vendimg{$vendorid}.") 4px no-repeat;' title='".ucfirst $vendorid."'>".ucfirst $vendorid."</td>\n"; }
#		else {
#			print "\t\t<td class='left vendimg' style='background: url(".$vendimg{unknown}.") 4px no-repeat;' title='".ucfirst $vendorid."'>".ucfirst $vendorid."</td>\n";
#		}
####

		if ($vendimg{$vendorid}) {
			print "\t\t<td class='left vendimg' style='background: url(".$vendimg{$vendorid}.") 4px no-repeat;' title='".ucfirst $vendorid."'></td>\n"; }
		else {
			print "\t\t<td class='left vendimg' style='background: url(".$vendimg{unknown}.") 4px no-repeat;' title='".ucfirst $vendorid."'></td>\n";
		}

		$shortname = substr($updatefile,rindex($updatefile,"/")+1);
		$shortname =~ s/(.*)_[\da-f]*(\.(exe|cab|psf)$)/$1_*$2/i;
		print <<END
		<td align='left' title='cache:/$updatefile'><a href="/updatecache/$updatefile">$shortname</a></td>
		<td align='right' >$filesize</td>
		<td align='center'>$filedate</td>
		<td align='center'>$lastaccess</td>
		<td align='center'>$lastcheck</td>
		<td align='center'><input id='frm$id' type='checkbox' name='TODELETE' value='$updatefile' title='$Lang::tr{'updxlrtr remove file'}' /></td>
	</tr>
END
;
	$id += 1;
	}
	print <<END
</tbody>
</table>
END
;
}

# -------------------------------------------------------------------

sub printlegenddownload
{
	print <<END
	<h4>$Lang::tr{'legend'}:</h4>
	<span class='ledblue' title='$Lang::tr{'updxlrtr condition download'}'>$Lang::tr{'updxlrtr condition download'}</span>
	<span class='ledgrey' title='$Lang::tr{'updxlrtr condition suspended'}'>$Lang::tr{'updxlrtr condition suspended'}</span>
	<span class='icodelete' title='$Lang::tr{'updxlrtr cancel download'}'>$Lang::tr{'updxlrtr cancel download'}</span>
END
;
}

# -------------------------------------------------------------------

sub printlegendicons
{
	print <<END
	<h4>$Lang::tr{'legend'}:</h4>
	<span class='icoreload' title='$Lang::tr{'updxlrtr last access'}'>$Lang::tr{'updxlrtr last access'}</span>
	<span class='icoglobe' title='$Lang::tr{'updxlrtr last checkup'}'>$Lang::tr{'updxlrtr last checkup'}</span>
	<span class='icodelete' title='$Lang::tr{'updxlrtr remove file'}'>$Lang::tr{'updxlrtr remove file'}</span>
END
;
}

# -------------------------------------------------------------------

sub printlegendstatus
{
	print <<END
	<h4>$Lang::tr{'status'}:</h4>
	<span class='ledgreen' title='$Lang::tr{'updxlrtr condition ok'}'>$Lang::tr{'updxlrtr condition ok'}</span>
	<span class='ledyellow' title='$Lang::tr{'updxlrtr condition nosource'}'>$Lang::tr{'updxlrtr condition nosource'}</span>
	<span class='ledred' title='$Lang::tr{'updxlrtr condition outdated'}'>$Lang::tr{'updxlrtr condition outdated'}</span>
	<span class='ledblue' title='$Lang::tr{'updxlrtr condition download'}'>$Lang::tr{'updxlrtr condition download'}</span>
	<span class='ledgrey' title='$Lang::tr{'updxlrtr condition unknown'}'>$Lang::tr{'updxlrtr condition unknown'}</span>
END
;
}

# -------------------------------------------------------------------

sub printlegendsource
{
	print "<h4>$Lang::tr{'updxlrtr source'}:</h4>";
	foreach my $name (sort keys %vendimg) {
		if ($name =~ /^unknown$/i) {
			print "<span class='vendimg' style='background: url($vendimg{$name}) no-repeat;' title='". ucfirst $Lang::tr{$name} ."'> ". ucfirst $Lang::tr{$name} ."</span> ";
		} else {
			print "<span class='vendimg' style='background: url($vendimg{$name}) no-repeat;' title='". ucfirst $name ."'> ". ucfirst $name ."</span> ";
		}
	}
}

# -------------------------------------------------------------------
# 2012-12-18:
# Seaching updbooster-image dir for all available images of format "updxl-src-<vendor>.gif"
# Build a hash in format '<vendor(name)>' -> '/images/updbooster/updxl-src-<vendor>.gif'

sub initvendimg {
	if (opendir(DIR, "$webhome$webimgdir")) {
		my @files = grep { /updxl-src-/ } readdir(DIR);
		my @vendor = ();
		closedir(DIR);
		foreach (@files) {
			@vendor = split (/[.-]/, $_);
			$vendimg{$vendor[2]} = "$webimgdir/$_";
		}
	}
	else {
			die "updxl: Can't access \"$webhome$webimgdir\". Error was: $!\n";
	}
}
# -------------------------------------------------------------------

sub printdiskusage {
	my $tabletitle = shift;
	my $tableid = shift;

	print <<END
<table id='$tableid'>
<caption>$tabletitle</caption>
<thead>
<tr>
	<th>$Lang::tr{'updxlrtr cache dir'}</th>
	<th>$Lang::tr{'size'}</th>
	<th>$Lang::tr{'used'}</th>
	<th>$Lang::tr{'free'}</th>
	<th>$Lang::tr{'percentage'}</th>
</tr>
</thead>
END
;
	open(DF,"/bin/df -h $repository|");
	@dfdata = <DF>;
	$dudata = `/usr/bin/du -hs $repository`;
	close DF;
	shift(@dfdata);
	chomp(@dfdata);
	chomp($dudata);
	$dfstr = join(' ',@dfdata);
	my ($device,$size,$used,$free,$percent,$mount) = split(' ',$dfstr);
	my ($duused,$tmp) = split(' ',$dudata);
	print <<END
<tbody>
<tr>
	<td>[$repository]</td>
	<td align='right'>$size</td>
	<td align='right'>$duused</td>
	<td align='right'>$free</td>
	<td>
END
;
	&percentbar($percent);
	print <<END
</td>
</tr>
</tbody>
</table>
END
;
}

# -------------------------------------------------------------------

sub savesettings
{

	if (($xlratorsettings{'ENABLE_AUTOCHECK'} eq 'on') && ($xlratorsettings{'AUTOCHECK_SCHEDULE'} eq 'daily'))
		{ system('/usr/local/bin/updxlratorctrl cron daily >/dev/null 2>&1'); }
	if (($xlratorsettings{'ENABLE_AUTOCHECK'} eq 'on') && ($xlratorsettings{'AUTOCHECK_SCHEDULE'} eq 'weekly'))
		{ system('/usr/local/bin/updxlratorctrl cron weekly >/dev/null 2>&1'); }
	if (($xlratorsettings{'ENABLE_AUTOCHECK'} eq 'on') && ($xlratorsettings{'AUTOCHECK_SCHEDULE'} eq 'monthly'))
	{ system('/usr/local/bin/updxlratorctrl cron monthly >/dev/null 2>&1'); }

	# don't save those variable to the settings file,
	# but we wan't to have them in the hash again after saving to file
	my $obsolete = $xlratorsettings{'REMOVE_OBSOLETE'};
	my $nosource = $xlratorsettings{'REMOVE_NOSOURCE'};
	my $outdated = $xlratorsettings{'REMOVE_OUTDATED'};
	my $todelete = $xlratorsettings{'REMOVE_TODELETE'};
	my $gui = $xlratorsettings{'EXTENDED_GUI'};

	delete($xlratorsettings{'REMOVE_OBSOLETE'});
	delete($xlratorsettings{'REMOVE_NOSOURCE'});
	delete($xlratorsettings{'REMOVE_OUTDATED'});
	delete($xlratorsettings{'REMOVE_TODELETE'});
	delete($xlratorsettings{'TODELETE'});
	delete($xlratorsettings{'EXTENDED_GUI'});

	&General::writehash("${General::swroot}/updatexlrator/settings", \%xlratorsettings);

	# put temp variables back into the hash
	$xlratorsettings{'REMOVE_OBSOLETE'} = $obsolete;
	$xlratorsettings{'REMOVE_NOSOURCE'} = $nosource;
	$xlratorsettings{'REMOVE_OUTDATED'} = $outdated;
	$xlratorsettings{'REMOVE_TODELETE'} = $todelete;
	$xlratorsettings{'EXTENDED_GUI'} = $gui;
}

# -------------------------------------------------------------------

sub percentbar {
	my $percent = shift;
	my $fg = "#a0a0a0";
	my $bg = "#e2e2e2";

	if ($percent =~ /^(\d+)%$/ ) {
		print <<END
	<div style='border: 1px solid $fg; width:100px; height:15px; background-color:$bg; float:left;'>
END
;
		if ($percent eq "100%")
			 { print "\t\t<div style='width:100%; height:13px; background-color:$fg; border: 1px solid $bg; margin: 0 auto 0 0;'></div>" }
		else
			 { print "\t\t<div style='width:$percent; height:13px; background-color:$fg; border: 1px solid $bg; margin: 0 auto 0 0;'></div>\n"; }
		print "</div> $percent";
	}
	return;
}

# -------------------------------------------------------------------

sub getmtime
{
	my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks) = stat($_[0]);
	return $mtime;
}

# -------------------------------------------------------------------

sub getPID
{
	my $pid='';
	my @psdata=`ps ax --no-heading`;

	foreach (@psdata)
		{ if (/$_[0]/) { ($pid)=/^\s*(\d+)/; } }
	return $pid;
}

# -------------------------------------------------------------------
